import io
import re
import sys


def ensure_unless_forced(passes_test, force_flag, message):
    """
    Checks the value of the test and outputs a message, either continuing or aborting processing
    depending on the value of the force flag.

    Helper function for the functions below that examine the output of different git commands.
    :param passes_test:   whether or not the test succeeded or failed
    :param force_flag:    whether to continue processing or abort
    :param message:       the message to display to the user
    :return:              None
    """
    if not passes_test:
        if force_flag:
            print(message + '; continuing due to force flag')
        else:
            print(message + '; aborting')
            sys.exit(1)


def ensure_tagged(c, force=False):
    """
    Ensures that the working copy is sitting at a final version tag, such as 3.5.2.

    :param c:      the invoke context
    :param force:  whether to force or not
    :return:       None
    """
    buf = io.StringIO()

    # this command outputs the nearest tag, but will also output
    # a tag plus a commit if you're a few commits from a tag;
    # this seems like useful behavior we could capitalize on later
    c.run('git describe --tags', out_stream=buf)
    ensure_unless_forced(re.match(r'^\d+\.\d+\.\d+$', buf.getvalue()), force, 'git working copy is not tagged')


def ensure_clean(c, force=False):
    """
    Ensures that the working copy has no changes or uncommitted files (i.e. is clean).

    :param c:       the invoke context
    :param force:   whether to force or not
    :return:        None
    """
    buf = io.StringIO()

    # the --porcelain version of different commands is intended for scripting
    # in this case, if we get no output, there is nothing interesting in
    # the working copy, which is the result we are hoping for;
    # if we do get output, then something is different or not committed
    # and thus the working directory is not clean.
    c.run('git status --porcelain', out_stream=buf, err_stream=buf)
    ensure_unless_forced(buf.getvalue() == '', force, 'git working copy is not clean')
